@@ -15,12 +15,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
 #include <goffice/goffice-config.h>
-#include <goffice/graph/gog-graph-impl.h>
+#include <goffice/graph/gog-graph.h>
 #include <goffice/graph/gog-renderer-svg.h>
 #include <goffice/graph/gog-renderer-impl.h>
 #include <goffice/graph/gog-style.h>
@@ -29,6 +29,7 @@
 #include <goffice/utils/go-font.h>
 #include <goffice/utils/go-marker.h>
 #include <goffice/utils/go-units.h>
+#include <goffice/utils/go-math.h>
 
 #include <gsf/gsf-libxml.h>
 #include <gsf/gsf-impl-utils.h>
@@ -41,10 +42,6 @@
 
 #define CC2XML(s) ((const xmlChar *)(s))
 
-#define GOG_RENDERER_SVG_TYPE	(gog_renderer_svg_get_type ())
-#define GOG_RENDERER_SVG(o)	(G_TYPE_CHECK_INSTANCE_CAST ((o), GOG_RENDERER_SVG_TYPE, GogRendererSvg))
-#define IS_GOG_RENDERER_SVG(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), GOG_RENDERER_SVG_TYPE))
-
 typedef struct _GogRendererSvg GogRendererSvg;
 
 struct _GogRendererSvg {
@@ -64,8 +61,6 @@ typedef GogRendererClass GogRendererSvgClass;
 
 static GObjectClass *parent_klass;
 
-static GType gog_renderer_svg_get_type (void);
-
 static void 
 set_double_prop (xmlNodePtr node, const char *name, double value)
 {
@@ -88,49 +83,6 @@ gog_renderer_svg_finalize (GObject *obj)
 	(*parent_klass->finalize) (obj);
 }
 
-static void
-gog_renderer_svg_clip_push (GogRenderer *rend, GogRendererClip *clip)
-{
-	GogRendererSvg *prend = GOG_RENDERER_SVG (rend);
-	char *buf;
-	xmlNodePtr child;
-	xmlNodePtr node;
-
-	prend->clip_counter++;
-	
-	node = xmlNewDocNode (prend->doc, NULL, CC2XML("clipPath"), NULL);
-	xmlAddChild (prend->defs, node);
-
-	buf = g_strdup_printf ("clip%i", prend->clip_counter);
-	xmlNewProp (node, CC2XML("id"), CC2XML(buf));
-	g_free (buf);
-	
-	child = xmlNewDocNode (prend->doc, NULL, CC2XML("rect"), NULL);
-	xmlAddChild (node, child);
-	
-	set_double_prop (child, "x", clip->area.x);
-	set_double_prop (child, "y", clip->area.y);
-	set_double_prop (child, "width", clip->area.w);
-	set_double_prop (child, "height", clip->area.h);
-	
-	node = xmlNewDocNode (prend->doc, NULL, CC2XML("g"), NULL);
-	xmlAddChild (prend->current_node, node);
-	
-	buf = g_strdup_printf ("url(#clip%i)", prend->clip_counter);
-	xmlNewProp (node, CC2XML ("clip-path"), CC2XML (buf));
-	g_free (buf);
-
-	prend->current_node = node;
-}
-
-static void
-gog_renderer_svg_clip_pop (GogRenderer *rend, GogRendererClip *clip)
-{
-	GogRendererSvg *prend = GOG_RENDERER_SVG (rend);
-	
-	prend->current_node = prend->current_node->parent;
-}
-
 static void
 draw_path (ArtVpath const *path, GString *string)
 {
@@ -156,6 +108,48 @@ draw_path (ArtVpath const *path, GString *string)
 		}
 }
 
+static void
+gog_renderer_svg_push_clip (GogRenderer *rend, GogRendererClip *clip)
+{
+	GogRendererSvg *prend = GOG_RENDERER_SVG (rend);
+	char *buf;
+	xmlNodePtr child;
+	xmlNodePtr node;
+	GString *string;
+
+	prend->clip_counter++;
+	
+	node = xmlNewDocNode (prend->doc, NULL, CC2XML("clipPath"), NULL);
+	xmlAddChild (prend->defs, node);
+
+	buf = g_strdup_printf ("clip%i", prend->clip_counter);
+	xmlNewProp (node, CC2XML("id"), CC2XML(buf));
+	g_free (buf);
+	
+	child = xmlNewChild (node, NULL, CC2XML ("path"), NULL);
+	string = g_string_new ("");
+	draw_path (clip->path, string);
+	xmlNewProp (child, CC2XML ("d"), CC2XML (string->str));
+	g_string_free (string, TRUE);
+	
+	node = xmlNewDocNode (prend->doc, NULL, CC2XML("g"), NULL);
+	xmlAddChild (prend->current_node, node);
+	
+	buf = g_strdup_printf ("url(#clip%i)", prend->clip_counter);
+	xmlNewProp (node, CC2XML ("clip-path"), CC2XML (buf));
+	g_free (buf);
+
+	prend->current_node = node;
+}
+
+static void
+gog_renderer_svg_pop_clip (GogRenderer *rend, GogRendererClip *clip)
+{
+	GogRendererSvg *prend = GOG_RENDERER_SVG (rend);
+	
+	prend->current_node = prend->current_node->parent;
+}
+
 static void
 stroke_dasharray (xmlNodePtr node, ArtVpathDash *dash)
 {
@@ -175,6 +169,8 @@ stroke_dasharray (xmlNodePtr node, ArtVpathDash *dash)
 	g_string_free (string, TRUE);
 }
 
+#define SVG_PATTERN_SCALE 2.0
+
 static void
 fill_properties (GogRenderer *renderer, xmlNodePtr node, gboolean narrow)
 {
@@ -196,9 +192,65 @@ fill_properties (GogRenderer *renderer, xmlNodePtr node, gboolean narrow)
 				opacity = color & 0xff;
 				if (opacity != 255) 
 					set_double_prop (node, "fill-opacity", (double) opacity / 255.0);
+			} else {
+				xmlNodePtr child, pat_node;
+				id = g_strdup (go_pattern_as_str (style->fill.pattern.pattern));
+				name = (char *) g_hash_table_lookup (prend->table, id);
+
+				if (!name) {
+					double height, width;
+					char *svg_path = go_pattern_get_svg_path (&style->fill.pattern, 
+										  &width, &height);
+					char buffer[G_ASCII_DTOSTR_BUF_SIZE];
+
+					if (svg_path == NULL) {
+						g_free (id);
+						break;
+					}
+
+					name = g_strdup (id);
+					g_hash_table_insert (prend->table, id, name);
+					pat_node = xmlNewChild (prend->defs, NULL, CC2XML ("pattern"), NULL);
+					xmlSetProp (pat_node, CC2XML ("x"), CC2XML ("0"));
+					xmlSetProp (pat_node, CC2XML ("y"), CC2XML ("0"));
+					set_double_prop (pat_node, "width", width * SVG_PATTERN_SCALE);
+					set_double_prop (pat_node, "height", height * SVG_PATTERN_SCALE);
+					xmlSetProp (pat_node, CC2XML ("id"), CC2XML (name));
+					xmlSetProp (pat_node, CC2XML ("patternUnits"), CC2XML ("userSpaceOnUse"));
+					
+					child = xmlNewChild (pat_node, NULL, CC2XML ("rect"), NULL);
+					xmlSetProp (child, CC2XML ("x"), CC2XML ("-0.1"));
+					xmlSetProp (child, CC2XML ("y"), CC2XML ("-0.1"));
+					set_double_prop (child, "width", width * SVG_PATTERN_SCALE + 0.2);
+					set_double_prop (child, "height", height * SVG_PATTERN_SCALE + 0.2);
+					g_ascii_dtostr (buffer, sizeof (buffer), 
+							(double) (style->fill.pattern.back & 0xff) / 255.0);
+					buf = g_strdup_printf ("stroke:none;fill:#%06x;fill-opacity:%s;",
+							       style->fill.pattern.back >> 8, buffer);
+					xmlSetProp (child, CC2XML ("style"), CC2XML (buf));
+					g_free (buf);
+
+					child = xmlNewChild (pat_node, NULL, CC2XML ("path"), NULL);
+					xmlSetProp (child, CC2XML ("d"), CC2XML (svg_path));
+					g_ascii_dtostr (buffer, sizeof (buffer), 
+							(double) (style->fill.pattern.fore & 0xff) / 255.0);
+					buf = g_strdup_printf ("stroke:none;fill:#%06x;fill-opacity:%s;", 
+							       style->fill.pattern.fore >> 8, buffer);
+					xmlSetProp (child, CC2XML ("style"), CC2XML (buf));
+					g_free (buf);
+					buf = g_strdup_printf ("scale(%g)", SVG_PATTERN_SCALE);
+					xmlSetProp (child, CC2XML ("transform"), CC2XML (buf)); 
+					g_free (buf);
+					g_free (svg_path);
+				} else {
+					g_free (id);
+				}
+				buf = g_strdup_printf ("url(#%s)", name);
+				xmlSetProp (node, CC2XML ("fill"), CC2XML (buf));
+				g_free (buf);
 			}
 			break;
-		}
+	        }
 
 		case GOG_FILL_STYLE_GRADIENT:
 			id = g_strdup_printf ("g_%x_%x_%x", style->fill.gradient.dir,
@@ -229,26 +281,26 @@ fill_properties (GogRenderer *renderer, xmlNodePtr node, gboolean narrow)
 				xmlNewProp (child, CC2XML ("gradientUnits"), CC2XML ("objectBoundingBox"));
 				switch (style->fill.gradient.dir % 4) {
 				case 0:
-					buf = (char*) "pad";
-					start = style->fill.pattern.fore;
-					end = style->fill.pattern.back;
-					break;
-				case 1:
 					buf = (char*) "pad";
 					start = style->fill.pattern.back;
 					end = style->fill.pattern.fore;
 					break;
-				case 2:
-					buf = (char*) "reflect";
+				case 1:
+					buf = (char*) "pad";
 					start = style->fill.pattern.fore;
 					end = style->fill.pattern.back;
+					break;
+				case 2:
+					buf = (char*) "reflect";
+					start = style->fill.pattern.back;
+					end = style->fill.pattern.fore;
 					x2 = x1 + (x2 - x1) / 2;
 					y2 = y1 + (y2 - y1) / 2;
 					break;
 				default:
 					buf = (char*) "reflect";
-					start = style->fill.pattern.back;
-					end = style->fill.pattern.fore;
+					start = style->fill.pattern.fore;
+					end = style->fill.pattern.back;
 					x2 = x1 + (x2 - x1) / 2;
 					y2 = y1 + (y2 - y1) / 2;
 					break;
@@ -312,8 +364,7 @@ fill_properties (GogRenderer *renderer, xmlNodePtr node, gboolean narrow)
 }
 
 static void
-gog_renderer_svg_draw_path (GogRenderer *renderer, ArtVpath const *path,
-			    GogViewAllocation const *bound)
+gog_renderer_svg_draw_path (GogRenderer *renderer, ArtVpath const *path)
 {
 	GogRendererSvg *prend = GOG_RENDERER_SVG (renderer);
 	GogStyle const *style = renderer->cur_style;
@@ -350,7 +401,7 @@ gog_renderer_svg_draw_path (GogRenderer *renderer, ArtVpath const *path,
 
 static void
 gog_renderer_svg_draw_polygon (GogRenderer *renderer, ArtVpath const *path, 
-			       gboolean narrow, GogViewAllocation const *bound)
+			       gboolean narrow)
 {
 	GogRendererSvg *prend = GOG_RENDERER_SVG (renderer);
 	GogStyle const *style = renderer->cur_style;
@@ -411,8 +462,7 @@ draw_bezier_path (ArtBpath const *path, GString *string)
 }
 
 static void
-gog_renderer_svg_draw_bezier_path (GogRenderer *rend, ArtBpath const *path,
-			       GogViewAllocation const *bound)
+gog_renderer_svg_draw_bezier_path (GogRenderer *rend, ArtBpath const *path)
 {
 	GogRendererSvg *prend = GOG_RENDERER_SVG (rend);
 	GogStyle const *style = rend->cur_style;
@@ -445,7 +495,7 @@ gog_renderer_svg_draw_bezier_path (GogRenderer *rend, ArtBpath const *path,
 
 static void
 gog_renderer_svg_draw_bezier_polygon (GogRenderer *renderer, ArtBpath const *path, 
-				      gboolean narrow, GogViewAllocation const *bound)
+				      gboolean narrow)
 {
 	GogRendererSvg *prend = GOG_RENDERER_SVG (renderer);
 	GogStyle const *style = renderer->cur_style;
@@ -488,7 +538,7 @@ gog_renderer_svg_draw_marker (GogRenderer *rend, double x, double y)
 	    (fill_path_raw == NULL))
 		return;
 
-	half_size = gog_renderer_line_size (rend, marker->size) / 2.0;
+	half_size = gog_renderer_line_size (rend, go_marker_get_size (marker)) / 2.0;
 	art_affine_scale (scaling, half_size, half_size);
 	art_affine_translate (translation, x, y);
 	art_affine_multiply (affine, scaling, translation);
@@ -503,11 +553,11 @@ gog_renderer_svg_draw_marker (GogRenderer *rend, double x, double y)
 	g_string_append_c (string, 'z');
 	xmlNewProp (node, CC2XML ("d"), CC2XML (string->str));
 	g_string_free (string, TRUE);
-	buf = g_strdup_printf ("#%06x", marker->fill_color >> 8);
+	buf = g_strdup_printf ("#%06x", go_marker_get_fill_color (marker) >> 8);
 	xmlNewProp (node, CC2XML ("fill"), CC2XML (buf));
 	g_free (buf);
 	xmlNewProp (node, CC2XML ("stroke"), CC2XML ("none"));
-	opacity = marker->fill_color & 0xff;
+	opacity = go_marker_get_fill_color (marker) & 0xff;
 	if (opacity != 255) 
 		set_double_prop (node, "fill-opacity", (double) opacity / 255.0);
 
@@ -522,10 +572,10 @@ gog_renderer_svg_draw_marker (GogRenderer *rend, double x, double y)
 	xmlNewProp (node, CC2XML ("stroke-linecap"), CC2XML ("round"));
 	set_double_prop (node, "stroke-width", 
 		gog_renderer_line_size (rend, go_marker_get_outline_width (marker)));
-	buf = g_strdup_printf ("#%06x", marker->outline_color >> 8);
+	buf = g_strdup_printf ("#%06x", go_marker_get_outline_color (marker) >> 8);
 	xmlNewProp (node, CC2XML ("stroke"), CC2XML (buf));
 	g_free (buf);
-	opacity = marker->outline_color & 0xff;
+	opacity = go_marker_get_outline_color (marker) & 0xff;
 	if (opacity != 255) 
 		set_double_prop (node, "stroke-opacity", (double) opacity / 255.0);
 
@@ -534,7 +584,7 @@ gog_renderer_svg_draw_marker (GogRenderer *rend, double x, double y)
 }
 
 static PangoLayout *
-make_layout (GogRenderer *rend, char const *text)
+make_layout (GogRenderer *rend, char const *text, gboolean use_markup)
 {
 	GogRendererSvg *prend = GOG_RENDERER_SVG (rend);
 	PangoLayout *layout;
@@ -560,75 +610,100 @@ make_layout (GogRenderer *rend, char const *text)
 	layout = pango_layout_new (prend->pango_context);
 	pango_layout_set_font_description (layout, fd);
 
-	pango_layout_set_text (layout, text, -1);
+	if (use_markup)
+		pango_layout_set_markup (layout, text, -1);
+	else
+		pango_layout_set_text (layout, text, -1);
 
 	return layout;
 }
 
 static void
-gog_renderer_svg_measure_text (GogRenderer *rend,
-			       char const *text, GogViewRequisition *size)
+gog_renderer_svg_get_text_OBR (GogRenderer *rend, char const *text, 
+			       gboolean use_markup, GOGeometryOBR *obr)
 {
 	PangoRectangle  rect;
-	PangoLayout    *layout = make_layout (rend, text);
+	PangoLayout    *layout = make_layout (rend, text, use_markup);
+	
 	pango_layout_get_pixel_extents (layout, NULL, &rect);
 	g_object_unref (layout);
-	size->w = gog_renderer_pt2r (rend, rect.width);
-	size->h = gog_renderer_pt2r (rend, rect.height);
+	obr->w = gog_renderer_pt2r (rend, rect.width);
+	obr->h = gog_renderer_pt2r (rend, rect.height);
 }
 
 static void
 gog_renderer_svg_draw_text (GogRenderer *rend, char const *text,
 			    GogViewAllocation const *pos, GtkAnchorType anchor,
-			    GogViewAllocation *result)
+			    gboolean use_markup)
 {
 	GogRendererSvg *prend = GOG_RENDERER_SVG (rend);
-	xmlNodePtr node;
-	char *buf;
-	double x, y;
-	int baseline;
+	GogStyle const *style = rend->cur_style;
 	PangoRectangle  rect;
-	PangoLayout* layout = make_layout (rend, "lp");
-	PangoFontDescription const *fd = rend->cur_style->font.font->desc;
+	PangoLayout* layout = make_layout (rend, text, use_markup);
+	PangoFontDescription const *fd = style->font.font->desc;
 	PangoLayoutIter* iter =pango_layout_get_iter(layout);
+	GOGeometryOBR obr;
+	GOGeometryAABR aabr;
+	GString *string;
+	xmlNodePtr node;
+	char buffer[G_ASCII_DTOSTR_BUF_SIZE];
+	char *buf;
+	double y_offset;
+	int baseline;
 
-	pango_layout_get_pixel_extents (layout, NULL, &rect);
-	x = pos->x;
-	/* adjust to the base line */
-	y = pos->y;
-	baseline = pango_layout_iter_get_baseline(iter);
-	pango_layout_iter_get_run_extents(iter, NULL, &rect);
-	y += gog_renderer_pt2r(rend, (baseline - rect.y) / PANGO_SCALE);
+	baseline = pango_layout_iter_get_baseline (iter);
+	pango_layout_iter_get_run_extents (iter, NULL, &rect);
 	pango_layout_iter_free(iter);
 	g_object_unref (layout);
+	obr.w = gog_renderer_pt2r (rend, rect.width / PANGO_SCALE);
+	obr.h = gog_renderer_pt2r (rend, rect.height / PANGO_SCALE);
+	obr.alpha = rend->cur_style->text_layout.angle * M_PI / 180.0;
+	obr.x = pos->x;
+	obr.y = pos->y;
+	go_geometry_OBR_to_AABR (&obr, &aabr);
+	y_offset = gog_renderer_pt2r(rend, (baseline - rect.y) / PANGO_SCALE) - obr.h / 2.0;
 
 	switch (anchor) {
-	case GTK_ANCHOR_CENTER : case GTK_ANCHOR_E : case GTK_ANCHOR_W :
-		y -= gog_renderer_pt2r(rend, (double) (rect.height / 2) / PANGO_SCALE);
-		break;
-	case GTK_ANCHOR_SE : case GTK_ANCHOR_S : case GTK_ANCHOR_SW :
-		y -= gog_renderer_pt2r(rend, (double)rect.height / PANGO_SCALE);
-		break;
-	default :
-		break;
+		case GTK_ANCHOR_NW: case GTK_ANCHOR_W: case GTK_ANCHOR_SW:
+			obr.x += aabr.w / 2.0;
+			break;
+		case GTK_ANCHOR_NE: case GTK_ANCHOR_E: case GTK_ANCHOR_SE:
+			obr.x -= aabr.w / 2.0;
+		default:
+			break;
 	}
-
+			
+	switch (anchor) {
+		case GTK_ANCHOR_NW: case GTK_ANCHOR_N: case GTK_ANCHOR_NE:
+			obr.y += aabr.h / 2.0;
+			break;
+		case GTK_ANCHOR_SW: case GTK_ANCHOR_S: case GTK_ANCHOR_SE:
+			obr.y -= aabr.h / 2.0;
+		default:
+			break;
+	}
+	
 	node = xmlNewDocNode (prend->doc, NULL, "text", NULL);
 	xmlNodeSetContent (node, CC2XML (text));
 	xmlAddChild (prend->current_node, node);
-	set_double_prop (node, "x", x);
-	set_double_prop (node, "y", y);
-	switch (anchor) {
-	case GTK_ANCHOR_CENTER : case GTK_ANCHOR_N : case GTK_ANCHOR_S :
-		xmlNewProp (node, CC2XML ("text-anchor"), CC2XML ("middle"));
-		break;
-	case GTK_ANCHOR_NE : case GTK_ANCHOR_SE : case GTK_ANCHOR_E :
-		xmlNewProp (node, CC2XML ("text-anchor"), CC2XML ("end"));
-		break;
-	default : break;
-	}
+	set_double_prop (node, "x", obr.x);
+	set_double_prop (node, "y", obr.y + y_offset);
+	xmlNewProp (node, CC2XML ("text-anchor"), CC2XML ("middle"));
+	string = g_string_new ("rotate(");
+	g_string_append (string, g_ascii_dtostr (buffer, sizeof (buffer), -rend->cur_style->text_layout.angle));
+	g_string_append_c (string, ',');
+	g_string_append (string, g_ascii_dtostr (buffer, sizeof (buffer), obr.x));
+	g_string_append_c (string, ',');
+	g_string_append (string, g_ascii_dtostr (buffer, sizeof (buffer), obr.y));
+	g_string_append_c (string, ')');
+	xmlNewProp (node, CC2XML ("transform"), CC2XML (string->str));
+       	g_string_free (string, TRUE);	
+	
+	buf = g_strdup_printf ("#%06x", style->font.color >> 8);
+	xmlNewProp (node, CC2XML ("fill"), CC2XML (buf));
+	g_free (buf);
 	xmlNewProp (node, CC2XML ("font-family"), CC2XML (pango_font_description_get_family (fd)));
-	buf = g_strdup_printf ("%d", (int)(rint (gog_renderer_pt2r(rend, pango_font_description_get_size (fd) / PANGO_SCALE))));
+	buf = g_strdup_printf ("%d", (int)(go_rint (gog_renderer_pt2r(rend, pango_font_description_get_size (fd) / PANGO_SCALE))));
 	xmlNewProp (node, CC2XML ("font-size"), CC2XML (buf));
 	g_free (buf);
 	switch (pango_font_description_get_weight (fd)) {
@@ -653,54 +728,22 @@ gog_renderer_svg_draw_text (GogRenderer *rend, char const *text,
 	}
 }
 
-static void
-gog_renderer_svg_class_init (GogRendererClass *rend_klass)
-{
-	GObjectClass *gobject_klass   = (GObjectClass *) rend_klass;
-
-	parent_klass = g_type_class_peek_parent (rend_klass);
-	gobject_klass->finalize	  	= gog_renderer_svg_finalize;
-	rend_klass->clip_push		= gog_renderer_svg_clip_push;
-	rend_klass->clip_pop	 	= gog_renderer_svg_clip_pop;
-	rend_klass->draw_path	  	= gog_renderer_svg_draw_path;
-	rend_klass->draw_polygon  	= gog_renderer_svg_draw_polygon;
-	rend_klass->draw_bezier_path 	= gog_renderer_svg_draw_bezier_path;
-	rend_klass->draw_bezier_polygon = gog_renderer_svg_draw_bezier_polygon;
-	rend_klass->draw_text	  	= gog_renderer_svg_draw_text;
-	rend_klass->draw_marker	  	= gog_renderer_svg_draw_marker;
-	rend_klass->measure_text  	= gog_renderer_svg_measure_text;
-}
-
-static GSF_CLASS (GogRendererSvg, gog_renderer_svg,
-		  gog_renderer_svg_class_init, NULL,
-		  GOG_RENDERER_TYPE)
-
-/**
- * gog_graph_export_to_svg :
- * @graph  : #GogGraph
- * @output : #GsfOutput
- * @width  :
- * @height :
- *
- * Renders @graph as SVG and stores it in @output.
- *
- * Returns TRUE on success.
- **/
-gboolean
-gog_graph_export_to_svg (GogGraph *graph, GsfOutput *output,
-			 double width, double height, double scale)
+static gboolean
+gog_renderer_svg_export_image (GogRenderer *renderer, GOImageFormat format,
+			       GsfOutput *output, double x_dpi, double y_dpi)
 {
 	GogViewAllocation allocation;
-	GogRendererSvg *prend;
+	GogRendererSvg *prend = GOG_RENDERER_SVG (renderer);
 	xmlNsPtr namespace;
+	double width_in_pts, height_in_pts;
 	gboolean success = TRUE;
 
-	gog_graph_force_update (graph);
+	if (format != GO_IMAGE_FORMAT_SVG) {
+		g_warning ("[GogRendererSVG::export_image] Unsupported format");
+		return FALSE;
+	}
 
-	prend = g_object_new (GOG_RENDERER_SVG_TYPE,
-			      "model", graph,
-			      NULL);
-	prend->base.scale = scale;
+	prend->base.scale = 1.0;
 	prend->doc = xmlNewDoc (CC2XML ("1.0"));
 
 	xmlNewDtd (prend->doc,
@@ -719,14 +762,15 @@ gog_graph_export_to_svg (GogGraph *graph, GsfOutput *output,
 
 	namespace = xmlNewNs (prend->doc->children, CC2XML ("http://www.w3.org/1999/xlink"), CC2XML ("xlink"));
 
-	set_double_prop (prend->doc->children, "width", width);
-	set_double_prop (prend->doc->children, "height", height);
+	gog_graph_get_size (renderer->model, &width_in_pts, &height_in_pts);
+	set_double_prop (prend->doc->children, "width", width_in_pts);
+	set_double_prop (prend->doc->children, "height", height_in_pts);
 
 	prend->clip_counter = 0;
 	allocation.x = 0.;
 	allocation.y = 0.;
-	allocation.w = width;
-	allocation.h = height;
+	allocation.w = width_in_pts;
+	allocation.h = height_in_pts;
 	gog_view_size_allocate (prend->base.view, &allocation);
 	gog_view_render	(prend->base.view, NULL);
 
@@ -740,8 +784,33 @@ gog_graph_export_to_svg (GogGraph *graph, GsfOutput *output,
 		success = FALSE;
 
 	xmlFreeDoc (prend->doc);
+	prend->doc = NULL;
 	g_hash_table_destroy (prend->table);
-	g_object_unref (prend);
+	prend->table = NULL;
 
 	return success;
 }
+
+static void
+gog_renderer_svg_class_init (GogRendererClass *rend_klass)
+{
+	GObjectClass *gobject_klass   = (GObjectClass *) rend_klass;
+
+	parent_klass = g_type_class_peek_parent (rend_klass);
+	gobject_klass->finalize	  	= gog_renderer_svg_finalize;
+	rend_klass->push_clip		= gog_renderer_svg_push_clip;
+	rend_klass->pop_clip	 	= gog_renderer_svg_pop_clip;
+	rend_klass->draw_path	  	= gog_renderer_svg_draw_path;
+	rend_klass->draw_polygon  	= gog_renderer_svg_draw_polygon;
+	rend_klass->draw_bezier_path 	= gog_renderer_svg_draw_bezier_path;
+	rend_klass->draw_bezier_polygon = gog_renderer_svg_draw_bezier_polygon;
+	rend_klass->draw_text	  	= gog_renderer_svg_draw_text;
+	rend_klass->draw_marker	  	= gog_renderer_svg_draw_marker;
+	rend_klass->get_text_OBR	= gog_renderer_svg_get_text_OBR;
+	rend_klass->export_image	= gog_renderer_svg_export_image;
+}
+
+GSF_CLASS (GogRendererSvg, gog_renderer_svg,
+	   gog_renderer_svg_class_init, NULL,
+	   GOG_RENDERER_TYPE)
+
