@@ -64,74 +64,12 @@ typedef struct {
 	int	 negative_fmt;	/* 0 - 3 */
 	int	 currency_symbol_index;
 	int	 list_element;
-	gboolean date_has_days;
-	gboolean date_has_months;
 	int      fraction_denominator;
 	gboolean simplify_mantissa;
 	int	 exponent_step;
 	gboolean use_markup;
 } GOFormatDetails;
 
-struct _GOFormat {
-	int		 ref_count;
-	char		*format;
-        GSList		*entries;	/* Of type GOFormatElement. */
-	GOFormatFamily	 family;
-	GOFormatDetails	 family_info;
-	gboolean	 is_var_width;
-	PangoAttrList	*markup;	/* only for GO_FORMAT_MARKUP */
-};
-
-GOFormat *go_format_new_from_XL		(char const *str, gboolean delocalize);
-GOFormat *go_format_new			(GOFormatFamily family,
-					 GOFormatDetails const *details);
-GOFormat *go_format_new_markup		(PangoAttrList *markup, gboolean add_ref);
-
-/* these do not add a reference to the result */
-GOFormat *go_format_general	   	(void);
-GOFormat *go_format_default_date	(void);
-GOFormat *go_format_default_time	(void);
-GOFormat *go_format_default_date_time	(void);
-GOFormat *go_format_default_percentage	(void);
-GOFormat *go_format_default_money	(void);
-
-char	 *go_format_str_delocalize	(char const *str);
-char   	 *go_format_str_as_XL	   	(char const *str, gboolean localized);
-char   	 *go_format_as_XL	   	(GOFormat const *fmt, gboolean localized);
-
-GOFormat *go_format_ref		 	(GOFormat *fmt);
-void      go_format_unref		(GOFormat *fmt);
-
-#define	  go_format_is_general(fmt)	((fmt)->family == GO_FORMAT_GENERAL)
-#define   go_format_is_markup(fmt)	((fmt)->family == GO_FORMAT_MARKUP)
-#define   go_format_is_text(fmt)	((fmt)->family == GO_FORMAT_TEXT)
-#define   go_format_is_var_width(fmt)	((fmt)->is_var_width)
-
-GOFormatNumberError go_format_value_gstring (GOFormat const *format,
-					     GString *res,
-					     double val,
-					     int col_width,
-					     GODateConventions const *date_conv,
-					     gboolean unicode_minus);
-char	*go_format_value		(GOFormat const *fmt, double val);
-
-gboolean go_format_eq			(GOFormat const *a, GOFormat const *b);
-GOFormat *go_format_inc_precision	(GOFormat const *fmt);
-GOFormat *go_format_dec_precision	(GOFormat const *fmt);
-GOFormat *go_format_toggle_1000sep	(GOFormat const *fmt);
-GOFormatFamily go_format_classify	(GOFormat const *fmt, GOFormatDetails *details);
-
-typedef struct {
-	gchar const *symbol;
-	gchar const *description;
-	gboolean precedes;
-	gboolean has_space;
-} GOFormatCurrency;
-
-/* Indexed by GOFormatFamily */
-GO_VAR_DECL char const * const * const go_format_builtins [];
-GO_VAR_DECL GOFormatCurrency     const go_format_currencies [];
-
 /*************************************************************************/
 
 typedef int (*GOFormatMeasure) (const GString *str, PangoLayout *layout);
@@ -156,41 +94,86 @@ void go_render_generall (PangoLayout *layout, GString *str,
 
 /*************************************************************************/
 
-typedef struct {
-	int  right_optional, right_spaces, right_req, right_allowed;
-	int  left_optional, left_spaces, left_req;
-	double scale;
-	gboolean rendered;
-	gboolean decimal_separator_seen;
-	gboolean group_thousands;
-	gboolean has_fraction;
-	gboolean unicode_minus;
+GOFormat *go_format_new_from_XL		(char const *str, gboolean delocalize);
+GOFormat *go_format_new			(GOFormatFamily family,
+					 GOFormatDetails const *details);
+GOFormat *go_format_new_markup		(PangoAttrList *markup, gboolean add_ref);
 
-	gboolean exponent_seen;
+/* these do not add a reference to the result */
+GOFormat *go_format_general	   	(void);
+GOFormat *go_format_default_date	(void);
+GOFormat *go_format_default_time	(void);
+GOFormat *go_format_default_date_time	(void);
+GOFormat *go_format_default_percentage	(void);
+GOFormat *go_format_default_money	(void);
 
-	int exponent_digit_nbr;
-	gboolean exponent_show_sign;
-	gboolean exponent_lower_e;
+char	 *go_format_str_delocalize	(char const *str);
+char   	 *go_format_str_as_XL	   	(char const *str, gboolean localized);
+char   	 *go_format_as_XL	   	(GOFormat const *fmt, gboolean localized);
 
-	gboolean use_markup;
-} GONumberFormat;
+GOFormat *go_format_ref		 	(GOFormat *fmt);
+void      go_format_unref		(GOFormat *fmt);
 
-void go_render_number (GString *result, double number, GONumberFormat const *info);
+gboolean  go_format_is_general          (GOFormat const *fmt);
+gboolean  go_format_is_markup           (GOFormat const *fmt);
+gboolean  go_format_is_text             (GOFormat const *fmt);
+gboolean  go_format_is_var_width        (GOFormat const *fmt);
+int       go_format_is_date             (GOFormat const *fmt);
+int       go_format_is_date_for_value   (GOFormat const *fmt, double val, char type);
 #ifdef GOFFICE_WITH_LONG_DOUBLE
-void go_render_numberl (GString *result, long double number, GONumberFormat const *info);
+int       go_format_is_date_for_valuel  (GOFormat const *fmt, long double val, char type);
 #endif
 
-/* Locale support routines */
-void	       go_set_untranslated_bools  (void);
-char const *   go_setlocale               (int category, char const *val);
-GString const *go_format_get_currency     (gboolean *precedes, gboolean *space_sep);
-gboolean       go_format_month_before_day (void);
-char           go_format_get_arg_sep      (void);
-char           go_format_get_col_sep      (void);
-char           go_format_get_row_sep      (void);
-GString const *go_format_get_thousand     (void);
-GString const *go_format_get_decimal      (void);
-char const *   go_format_boolean          (gboolean b);
+GOFormatFamily         go_format_get_family (GOFormat const *fmt);
+const GOFormatDetails *go_format_get_details (GOFormat const *fmt);
+
+const PangoAttrList *go_format_get_markup (GOFormat const *fmt);
+
+gboolean go_format_is_simple (GOFormat const *fmt);
+
+GOFormatNumberError
+go_format_value_gstring (PangoLayout *layout, GString *str,
+			 const GOFormatMeasure measure,
+			 const GOFontMetrics *metrics,
+			 GOFormat const *format,
+			 double val, char type, const char *sval,
+			 GOColor *go_color,
+			 int col_width,
+			 GODateConventions const *date_conv,
+			 gboolean unicode_minus);
+char	*go_format_value (GOFormat const *fmt, double val);
+#ifdef GOFFICE_WITH_LONG_DOUBLE
+GOFormatNumberError
+go_format_value_gstringl (PangoLayout *layout, GString *str,
+			  const GOFormatMeasure measure,
+			  const GOFontMetrics *metrics,
+			  GOFormat const *format,
+			  long double val, char type, const char *sval,
+			  GOColor *go_color,
+			  int col_width,
+			  GODateConventions const *date_conv,
+			  gboolean unicode_minus);
+char	*go_format_valuel (GOFormat const *fmt, long double val);
+#endif
+
+gboolean go_format_eq			(GOFormat const *a, GOFormat const *b);
+GOFormat *go_format_inc_precision	(GOFormat const *fmt);
+GOFormat *go_format_dec_precision	(GOFormat const *fmt);
+GOFormat *go_format_toggle_1000sep	(GOFormat const *fmt);
+GOFormatFamily go_format_classify	(GOFormat const *fmt, GOFormatDetails *details);
+
+typedef struct {
+	gchar const *symbol;
+	gchar const *description;
+	gboolean precedes;
+	gboolean has_space;
+} GOFormatCurrency;
+
+/* Indexed by GOFormatFamily */
+GO_VAR_DECL char const * const * const go_format_builtins [];
+GO_VAR_DECL GOFormatCurrency     const go_format_currencies [];
+
+/*************************************************************************/
 
 void go_number_format_init (void);
 void go_number_format_shutdown (void);
