@@ -336,8 +336,8 @@ gog_polar_plot_axis_get_bounds (GogPlot *plot, GogAxisType axis,
 
 	switch (axis) {
 	case GOG_AXIS_CIRCULAR:
-		bounds->val.minima = bounds->logical.minima= 0.;
-		bounds->val.maxima = bounds->logical.maxima= 360.0;
+		bounds->val.minima = bounds->logical.minima= -G_MAXDOUBLE;
+		bounds->val.maxima = bounds->logical.maxima=  G_MAXDOUBLE;
 		bounds->is_discrete    = FALSE;
 		break;
 	case GOG_AXIS_RADIAL:
@@ -386,12 +386,6 @@ GSF_DYNAMIC_CLASS (GogPolarPlot, gog_polar_plot,
 typedef GogPlotView		GogRTView;
 typedef GogPlotViewClass	GogRTViewClass;
 
-static double
-fmin (double a, double b)
-{
-	return (a < b) ? a : b;
-}
-
 static void
 gog_rt_view_render (GogView *view, GogViewAllocation const *bbox)
 {
@@ -404,7 +398,7 @@ gog_rt_view_render (GogView *view, GogViewAllocation const *bbox)
 	GogViewAllocation const *area;
 	GSList   *ptr;
 	ArtVpath *path, *clip_path;
-	ArtBpath *bpath;
+	ArtBpath *bpath = NULL;
 	double th0, theta_min, theta_max, theta;
 	double rho_min, rho_max, rho;
 	gboolean const is_area = GOG_IS_PLOT_RADAR_AREA (model);
@@ -495,9 +489,10 @@ gog_rt_view_render (GogView *view, GogViewAllocation const *bbox)
 		if (is_polar) { 
 			bpath = gog_renderer_get_ring_wedge_bpath (parms->cx, parms->cy, parms->rx, parms->ry,
 								   0.0, 0.0, -parms->th0, -parms->th1);
-			clip_path = art_bez_path_to_vec (bpath, .1);
-			g_free (bpath);
-			gog_renderer_push_clip (view->renderer, clip_path);	
+			if (bpath != NULL) {
+				clip_path = art_bez_path_to_vec (bpath, .1);
+				gog_renderer_push_clip (view->renderer, clip_path);	
+			}
 		}
 
 		if (closed_shape && is_area) {
@@ -505,35 +500,21 @@ gog_rt_view_render (GogView *view, GogViewAllocation const *bbox)
 		} else
 			gog_renderer_draw_path (view->renderer, path);
 
-		if (is_polar)
+		if (is_polar && bpath != NULL) {
+			g_free (bpath);
 			gog_renderer_pop_clip (view->renderer);
+		}
 
 		gog_renderer_pop_style (view->renderer);
 	}
 	gog_chart_map_free (chart_map);
 }
 
-static gboolean
-gog_rt_view_info_at_point (GogView *view, double x, double y,
-			      GogObject const *cur_selection,
-			      GogObject **obj, char **name)
-{
-	double radius = fmin (view->allocation.h, view->allocation.w)/2.0;
-
-	x -= view->allocation.x + view->allocation.w/2.;
-	y -= view->allocation.y + view->allocation.h/2.;
-	if ((x*x + y*y) > (radius*radius))
-		return FALSE;
-	
-	return TRUE;
-}
-
 static void
 gog_rt_view_class_init (GogViewClass *view_klass)
 {
 	view_klass->render	  = gog_rt_view_render;
-	view_klass->info_at_point = gog_rt_view_info_at_point;
-	view_klass->clip	  = TRUE;
+	view_klass->clip	  = FALSE;
 }
 
 GSF_DYNAMIC_CLASS (GogRTView, gog_rt_view,
