@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -27,6 +27,8 @@
 #include <goffice/graph/gog-theme.h>
 #include <goffice/graph/gog-axis.h>
 #include <goffice/graph/gog-error-bar.h>
+#include <goffice/graph/gog-chart.h>
+#include <goffice/graph/gog-series-lines.h>
 #include <goffice/data/go-data.h>
 #include <goffice/utils/go-color.h>
 #include <goffice/utils/go-marker.h>
@@ -35,10 +37,10 @@
 #include <goffice/utils/go-line.h>
 #include <goffice/app/module-plugin-defs.h>
 
-#include <glib/gi18n.h>
-#include <gtk/gtklabel.h>
+#include <glib/gi18n-lib.h>
 #include <gsf/gsf-impl-utils.h>
 #include <math.h>
+#include <string.h>
 
 typedef struct {
 	GogPlotClass	base;
@@ -50,6 +52,8 @@ typedef Gog2DPlotClass GogXYPlotClass;
 
 typedef Gog2DPlotClass GogBubblePlotClass;
 
+typedef Gog2DPlotClass GogXYColorPlotClass;
+
 GOFFICE_PLUGIN_MODULE_HEADER;
 
 static GogObjectClass *plot2d_parent_klass;
@@ -163,24 +167,6 @@ gog_2d_plot_adjust_bounds (Gog2DPlot *model, double *x_min, double *x_max, doubl
 	klass->adjust_bounds (model, x_min, x_max, y_min, y_max);
 }
 
-static GogAxisSet
-gog_2d_plot_axis_set_pref (GogPlot const *plot)
-{
-	return GOG_AXIS_SET_XY;
-}
-
-static gboolean
-gog_2d_plot_axis_set_is_valid (GogPlot const *plot, GogAxisSet type)
-{
-	return type == GOG_AXIS_SET_XY;
-}
-
-static gboolean
-gog_2d_plot_axis_set_assign (GogPlot *plot, GogAxisSet type)
-{
-	return type == GOG_AXIS_SET_XY;
-}
-
 static GOData *
 gog_2d_plot_axis_get_bounds (GogPlot *plot, GogAxisType axis,
 			     GogPlotBoundInfo *bounds)
@@ -241,11 +227,9 @@ gog_2d_plot_class_init (GogPlotClass *plot_klass)
 
 	plot_klass->desc.num_series_min = 1;
 	plot_klass->desc.num_series_max = G_MAXINT;
-	plot_klass->series_type  = gog_xy_series_get_type ();
-	plot_klass->axis_set_pref     = gog_2d_plot_axis_set_pref;
-	plot_klass->axis_set_is_valid = gog_2d_plot_axis_set_is_valid;
-	plot_klass->axis_set_assign   = gog_2d_plot_axis_set_assign;
-	plot_klass->axis_get_bounds   = gog_2d_plot_axis_get_bounds;
+	plot_klass->series_type  	= gog_xy_series_get_type ();
+	plot_klass->axis_set	      	= GOG_AXIS_SET_XY;
+	plot_klass->axis_get_bounds   	= gog_2d_plot_axis_get_bounds;
 }
 
 static void
@@ -263,7 +247,8 @@ enum {
 	GOG_XY_PROP_0,
 	GOG_XY_PROP_DEFAULT_STYLE_HAS_MARKERS,
 	GOG_XY_PROP_DEFAULT_STYLE_HAS_LINES,
-	GOG_XY_PROP_USE_SPLINES
+	GOG_XY_PROP_USE_SPLINES,
+	GOG_XY_PROP_INTERPOLATION,
 };
 
 static GogObjectClass *xy_parent_klass;
@@ -288,12 +273,19 @@ gog_xy_set_property (GObject *obj, guint param_id,
 	case GOG_XY_PROP_DEFAULT_STYLE_HAS_MARKERS:
 		xy->default_style_has_markers = g_value_get_boolean (value);
 		break;
-	case GOG_XY_PROP_DEFAULT_STYLE_HAS_LINES:
+	case GOG_XY_PROP_DEFAULT_STYLE_HAS_LINES: {
 		xy->default_style_has_lines = g_value_get_boolean (value);
 		break;
+	}
 	case GOG_XY_PROP_USE_SPLINES:
-		xy->use_splines = g_value_get_boolean (value);
+		if (g_value_get_boolean (value))
+			xy->interpolation = GO_LINE_INTERPOLATION_SPLINE;
 		break;
+	case GOG_XY_PROP_INTERPOLATION: {
+		char const *s = g_value_get_string (value);
+		xy->interpolation = go_line_interpolation_from_str (s);;
+		break;
+	}
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
 	}
@@ -311,7 +303,10 @@ gog_xy_get_property (GObject *obj, guint param_id,
 		g_value_set_boolean (value, xy->default_style_has_lines);
 		break;
 	case GOG_XY_PROP_USE_SPLINES:
-		g_value_set_boolean (value, xy->use_splines);
+		g_value_set_boolean (value, xy->interpolation == GO_LINE_INTERPOLATION_SPLINE);
+		break;
+	case GOG_XY_PROP_INTERPOLATION:
+		g_value_set_string (value, go_line_interpolation_as_str (xy->interpolation));
 		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
@@ -341,7 +336,10 @@ gog_xy_plot_class_init (GogPlotClass *plot_klass)
 		g_param_spec_boolean ("use-splines", NULL,
 			"Should the plot use splines instead of linear interpolation",
 			FALSE, G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
-
+	g_object_class_install_property (gobject_klass, GOG_XY_PROP_INTERPOLATION,
+		g_param_spec_string  ("interpolation", NULL,
+			_("Interpolation type (none, linear, spline or step) with variant, if any."),
+            "none", G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	gog_klass->type_name	= gog_xy_plot_type_name;
 
 	{
@@ -362,7 +360,7 @@ gog_xy_plot_class_init (GogPlotClass *plot_klass)
 		};
 		plot_klass->desc.series.dim = dimensions;
 		plot_klass->desc.series.num_dim = G_N_ELEMENTS (dimensions);
-		plot_klass->desc.series.style_fields = GOG_STYLE_LINE | GOG_STYLE_MARKER;
+		plot_klass->desc.series.style_fields = GOG_STYLE_LINE | GOG_STYLE_MARKER | GOG_STYLE_INTERPOLATION;
 	}
 }
 
@@ -555,6 +553,191 @@ GSF_DYNAMIC_CLASS (GogBubblePlot, gog_bubble_plot,
 	gog_bubble_plot_class_init, gog_bubble_plot_init,
 	GOG_2D_PLOT_TYPE)
 
+/*****************************************************************************/
+
+enum {
+	GOG_XY_COLOR_PROP_0,
+	GOG_XY_COLOR_PROP_DEFAULT_STYLE_HAS_LINES,
+	GOG_XY_COLOR_PROP_INTERPOLATION,
+};
+
+static GogObjectClass *map_parent_klass;
+
+#define GOG_XY_COLOR_PLOT_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), GOG_XY_COLOR_PLOT_TYPE, GogXYColorPlotClass))
+
+static void
+gog_xy_color_plot_clear_formats (GogXYColorPlot *map)
+{
+	if (map->z.fmt != NULL) {
+		go_format_unref (map->z.fmt);
+		map->z.fmt = NULL;
+	}
+}
+
+static void
+gog_xy_color_plot_update (GogObject *obj)
+{
+	GogXYColorPlot *model = GOG_XY_COLOR_PLOT (obj);
+	GogXYSeries const *series = NULL;
+	double z_min, z_max, tmp_min, tmp_max;
+	GSList *ptr;
+
+	z_min = DBL_MAX;
+	z_max = -DBL_MAX;
+	gog_xy_color_plot_clear_formats (model);
+	for (ptr = model->base.base.series ; ptr != NULL ; ptr = ptr->next) {
+		series = ptr->data;
+		if (!gog_series_is_valid (GOG_SERIES (series)))
+			continue;
+
+		go_data_vector_get_minmax (GO_DATA_VECTOR (
+			series->base.values[2].data), &tmp_min, &tmp_max);
+		if (z_min > tmp_min) z_min = tmp_min;
+		if (z_max < tmp_max) z_max = tmp_max;
+		if (model->z.fmt == NULL)
+			model->z.fmt = go_data_preferred_fmt (series->base.values[2].data);
+	}
+
+	if (model->z.minima != z_min || model->z.maxima != z_max) {
+		model->z.minima = z_min;
+		model->z.maxima = z_max;
+		gog_axis_bound_changed (model->base.base.axis[GOG_AXIS_COLOR], GOG_OBJECT (model));
+	}
+	map_parent_klass->update (obj);
+}
+
+static GOData *
+gog_xy_color_plot_axis_get_bounds (GogPlot *plot, GogAxisType axis,
+			     GogPlotBoundInfo *bounds)
+{
+	if (axis == GOG_AXIS_PSEUDO_3D) {
+		GogXYColorPlot *model = GOG_XY_COLOR_PLOT (plot);
+
+		bounds->val.minima = model->z.minima;
+		bounds->val.maxima = model->z.maxima;
+		bounds->is_discrete = model->z.minima > model->z.maxima ||
+			!go_finite (model->z.minima) ||
+			!go_finite (model->z.maxima);
+		if (bounds->fmt == NULL && model->z.fmt != NULL)
+			bounds->fmt = go_format_ref (model->z.fmt);
+		return NULL;
+	}
+	return GOG_PLOT_CLASS (map_parent_klass)->axis_get_bounds (plot, axis, bounds);
+}
+
+static char const *
+gog_xy_color_plot_type_name (G_GNUC_UNUSED GogObject const *item)
+{
+	/* xgettext : the base for how to name map like plot objects
+	 * eg The 2nd plot in a chart will be called
+	 * 	Map2 */
+	return N_("XYColor");
+}
+
+static void
+gog_xy_color_plot_set_property (GObject *obj, guint param_id,
+		     GValue const *value, GParamSpec *pspec)
+{
+	GogXYColorPlot *map = GOG_XY_COLOR_PLOT (obj);
+	switch (param_id) {
+	case GOG_XY_COLOR_PROP_DEFAULT_STYLE_HAS_LINES: {
+		map->default_style_has_lines = g_value_get_boolean (value);
+		break;
+	}
+	case GOG_XY_COLOR_PROP_INTERPOLATION: {
+		char const *s = g_value_get_string (value);
+		map->interpolation = go_line_interpolation_from_str (s);;
+		break;
+	}
+	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
+		 break;
+	}
+}
+static void
+gog_xy_color_plot_get_property (GObject *obj, guint param_id,
+		     GValue *value, GParamSpec *pspec)
+{
+	GogXYColorPlot const *map = GOG_XY_COLOR_PLOT (obj);
+	switch (param_id) {
+	case GOG_XY_COLOR_PROP_DEFAULT_STYLE_HAS_LINES:
+		g_value_set_boolean (value, map->default_style_has_lines);
+		break;
+	case GOG_XY_COLOR_PROP_INTERPOLATION:
+		g_value_set_string (value, go_line_interpolation_as_str (map->interpolation));
+		break;
+	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
+		 break;
+	}
+}
+
+static void
+gog_xy_color_plot_finalize (GObject *obj)
+{
+	gog_xy_color_plot_clear_formats (GOG_XY_COLOR_PLOT (obj));
+	G_OBJECT_CLASS (map_parent_klass)->finalize (obj);
+}
+
+static void
+gog_xy_color_plot_class_init (GogPlotClass *plot_klass)
+{
+	GObjectClass *gobject_klass = (GObjectClass *) plot_klass;
+	GogObjectClass *gog_klass = (GogObjectClass *) plot_klass;
+
+	map_parent_klass = g_type_class_peek_parent (plot_klass);
+
+	gobject_klass->set_property = gog_xy_color_plot_set_property;
+	gobject_klass->get_property = gog_xy_color_plot_get_property;
+	gobject_klass->finalize     = gog_xy_color_plot_finalize;
+
+	g_object_class_install_property (gobject_klass, GOG_XY_COLOR_PROP_DEFAULT_STYLE_HAS_LINES,
+		g_param_spec_boolean ("default-style-has-lines", NULL,
+			"Should the default style of a series include lines",
+			TRUE, G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+	g_object_class_install_property (gobject_klass, GOG_XY_COLOR_PROP_INTERPOLATION,
+		g_param_spec_string  ("interpolation", NULL,
+			_("Interpolation type (none, linear, spline or step) with variant, if any."),
+            "none", G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+	gog_klass->type_name	= gog_xy_color_plot_type_name;
+	gog_klass->update		= gog_xy_color_plot_update;
+
+	{
+		static GogSeriesDimDesc dimensions[] = {
+			{ N_("X"), GOG_SERIES_SUGGESTED, FALSE,
+			  GOG_DIM_INDEX, GOG_MS_DIM_CATEGORIES },
+			{ N_("Y"), GOG_SERIES_REQUIRED, FALSE,
+			  GOG_DIM_VALUE, GOG_MS_DIM_VALUES },
+			{ N_("Z"), GOG_SERIES_REQUIRED, FALSE,
+			  GOG_DIM_VALUE, GOG_MS_DIM_EXTRA1 },
+/* Names of the error data are not translated since they are not used */
+			{ "Y+err", GOG_SERIES_ERRORS, FALSE,
+			  GOG_DIM_VALUE, GOG_MS_DIM_ERR_plus1 },
+			{ "Y-err", GOG_SERIES_ERRORS, FALSE,
+			  GOG_DIM_VALUE, GOG_MS_DIM_ERR_minus1 },
+			{ "X+err", GOG_SERIES_ERRORS, FALSE,
+			  GOG_DIM_VALUE, GOG_MS_DIM_ERR_plus2 },
+			{ "X-err", GOG_SERIES_ERRORS, FALSE,
+			  GOG_DIM_VALUE, GOG_MS_DIM_ERR_minus2 }
+		};
+		plot_klass->desc.series.dim = dimensions;
+		plot_klass->desc.series.num_dim = G_N_ELEMENTS (dimensions);
+		plot_klass->desc.series.style_fields = GOG_STYLE_LINE | GOG_STYLE_MARKER
+			| GOG_STYLE_INTERPOLATION |GOG_STYLE_MARKER_NO_COLOR;
+	}
+	plot_klass->axis_set	      	= GOG_AXIS_SET_XY_COLOR;
+	plot_klass->axis_get_bounds   	= gog_xy_color_plot_axis_get_bounds;
+}
+
+static void
+gog_xy_color_plot_init (GogXYColorPlot *map)
+{
+	map->default_style_has_lines = FALSE;
+}
+
+GSF_DYNAMIC_CLASS (GogXYColorPlot, gog_xy_color_plot,
+	gog_xy_color_plot_class_init, gog_xy_color_plot_init,
+	GOG_2D_PLOT_TYPE)
+
+
 /*****************************************************************************/
 typedef GogPlotView		GogXYView;
 typedef GogPlotViewClass	GogXYViewClass;
@@ -570,7 +753,6 @@ bubble_draw_circle (GogView *view, double x, double y, double radius)
 	path[0].x = path[MAX_ARC_SEGMENTS].x = x + radius;
 	path[0].y = path[MAX_ARC_SEGMENTS].y = y;
 	path[0].code = ART_MOVETO;
-#warning what about small bubbles. With a very small radius, libart emits lot of warnings.
 	if (radius < 1.) radius = 1.;
 	for (i = 1, theta = dt; i < MAX_ARC_SEGMENTS; i++, theta += dt) {
 		path[i].x = x + radius * cos (theta);
@@ -580,11 +762,28 @@ bubble_draw_circle (GogView *view, double x, double y, double radius)
 	}
 	path[MAX_ARC_SEGMENTS].code = ART_LINETO;
 	path[MAX_ARC_SEGMENTS + 1].code = ART_END;
-	gog_renderer_draw_polygon (view->renderer, path, FALSE, &view->residual);
+	gog_renderer_draw_polygon (view->renderer, path, FALSE);
+}
+
+static GOColor
+get_map_color (double z)
+{
+	if (z <= 0.)
+		return RGBA_BLUE;
+	if (z <= 1.)
+		return RGBA_BLUE + ((int) (z * 255.) << 16);
+	if (z <= 2.)
+		return RGBA_GREEN + ((int) ((2. - z) * 255) << 8);
+	if (z <= 4.)
+		return RGBA_GREEN + ((int) ((z / 2. - 1.) * 255) << 24);
+	if (z <= 6.)
+		return RGBA_RED + ((int) ((3. - z / 2.) * 255) << 16);
+	return RGBA_RED;
 }
 
 typedef struct {
 	double x, y;
+	GOColor color;
 } MarkerData;
 
 static void
@@ -592,37 +791,57 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 {
 	Gog2DPlot const *model = GOG_2D_PLOT (view->model);
 	unsigned num_series;
-	GogAxisMap *x_map, *y_map;
+	GogChart *chart = GOG_CHART (view->model->parent);
+	GogChartMap *chart_map;
+	GogAxisMap *x_map, *y_map, *z_map;
 	GogXYSeries const *series = NULL;
 	unsigned i ,j ,k ,n, tmp;
 	GogTheme *theme = gog_object_get_theme (GOG_OBJECT (model));
 	GogStyle *neg_style = NULL;
+	GogViewAllocation const *area;
 	GSList *ptr;
 	double const *y_vals, *x_vals = NULL, *z_vals = NULL;
-	double x = 0., y = 0., z, x_canvas = 0., y_canvas = 0.;
-	double zmax, rmax = 0.;
+	double x = 0., y = 0., z = 0., x_canvas = 0., y_canvas = 0.;
+	double zmax, rmax = 0., x_zero, y_zero;
 	double x_margin_min, x_margin_max, y_margin_min, y_margin_max, margin;
 	double xerrmin, xerrmax, yerrmin, yerrmax;
 	GogStyle *style = NULL;
 	gboolean show_marks, show_lines, show_negatives, in_3d, size_as_area = TRUE;
+	gboolean is_map = GOG_IS_XY_COLOR_PLOT (model);
 
 	MarkerData **markers;
 	unsigned *num_markers;
 
-	x_map = gog_axis_map_new (model->base.axis[0], 
-				  view->residual.x , view->residual.w);
-	y_map = gog_axis_map_new (model->base.axis[1], 
-				  view->residual.y + view->residual.h, 
-				  -view->residual.h);
-
-	if (!(gog_axis_map_is_valid (x_map) &&
-	      gog_axis_map_is_valid (y_map))) {
-		gog_axis_map_free (x_map);
-		gog_axis_map_free (y_map);
+	area = gog_chart_view_get_plot_area (view->parent);
+	chart_map = gog_chart_map_new (chart, area, 
+				       GOG_PLOT (model)->axis[GOG_AXIS_X], 
+				       GOG_PLOT (model)->axis[GOG_AXIS_Y],
+				       NULL, FALSE);
+	if (!gog_chart_map_is_valid (chart_map)) {
+		gog_chart_map_free (chart_map);
 		return;
 	}
 
-	gog_renderer_clip_push (view->renderer, &view->allocation);
+	x_map = gog_chart_map_get_axis_map (chart_map, 0);
+	y_map = gog_chart_map_get_axis_map (chart_map, 1);
+	z_map = is_map?
+			gog_axis_map_new (model->base.axis[GOG_AXIS_COLOR], 0, 6):
+			NULL;
+
+	/* Draw drop lines from point to axis start. To change this behaviour
+	 * and draw drop lines from point to zero, we can use gog_axis_map_get_baseline:
+	 * x_zero = gog_axis_map_get_baseline (x_map); 
+	 * What we really want is to draw drop lines from point to
+	 * a selected axis. But for this purpose, we need a GogAxisBase selector in
+	 * GogSeriesLine, which doesn't really know what it's supposed to do with it. */
+
+	gog_axis_map_get_extents (x_map, &x_zero, NULL); 
+	x_zero = gog_axis_map_to_view (x_map, x_zero);
+	gog_axis_map_get_extents (y_map, &y_zero, NULL); 
+	y_zero = gog_axis_map_to_view (y_map, y_zero);
+
+	gog_renderer_push_clip (view->renderer, 
+				gog_renderer_get_rectangle_vpath (&view->allocation));
 
 	for (num_series = 0, ptr = model->base.series ; ptr != NULL ; ptr = ptr->next, num_series++);
 	markers = g_alloca (num_series * sizeof (MarkerData *));
@@ -652,6 +871,50 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 		if (n <= 0)
 			continue;
 
+		/* plot drop lines if any */
+		if (series->hdroplines) {
+			ArtVpath droppath[3];
+			droppath[0].code = ART_MOVETO;
+			droppath[1].code = ART_LINETO;
+			droppath[2].code = ART_END;
+			droppath[0].x = x_zero;
+			gog_renderer_push_style (view->renderer,
+				gog_styled_object_get_style (GOG_STYLED_OBJECT (series->hdroplines)));
+			for (i = 0; i < n; i++) {
+				if (!gog_axis_map_finite (y_map, y_vals[i]))
+					continue;
+				x = x_vals ? x_vals[i] : i + 1;
+				if (!gog_axis_map_finite (x_map, x))
+					continue;
+				droppath[1].x = gog_axis_map_to_view (x_map, x);
+				droppath[0].y = droppath[1].y = gog_axis_map_to_view (y_map, y_vals[i]);
+				gog_series_lines_render (GOG_SERIES_LINES (series->hdroplines),
+								view->renderer, bbox, droppath, FALSE);
+			}
+			gog_renderer_pop_style (view->renderer);
+		}
+		if (series->vdroplines) {
+			ArtVpath droppath[3];
+			droppath[0].code = ART_MOVETO;
+			droppath[1].code = ART_LINETO;
+			droppath[2].code = ART_END;
+			droppath[0].y = y_zero;
+			gog_renderer_push_style (view->renderer,
+				gog_styled_object_get_style (GOG_STYLED_OBJECT (series->vdroplines)));
+			for (i = 0; i < n; i++) {
+				if (!gog_axis_map_finite (y_map, y_vals[i]))
+					continue;
+				x = x_vals ? x_vals[i] : i + 1;
+				if (!gog_axis_map_finite (x_map, x))
+					continue;
+				droppath[1].y = gog_axis_map_to_view (y_map, y_vals[i]);
+				droppath[0].x = droppath[1].x = gog_axis_map_to_view (x_map, x);
+				gog_series_lines_render (GOG_SERIES_LINES (series->vdroplines),
+								view->renderer, bbox, droppath, FALSE);
+			}
+			gog_renderer_pop_style (view->renderer);
+		}
+
 		show_marks = gog_style_is_marker_visible (style);
 		show_lines = gog_style_is_line_visible (style);
 		if (!show_marks && !show_lines)
@@ -661,6 +924,14 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 			style = gog_style_dup (style);
 		gog_renderer_push_style (view->renderer, style);
 
+		if (is_map) {
+			z_vals = go_data_vector_get_values (
+				GO_DATA_VECTOR (series->base.values[2].data));
+			tmp = go_data_vector_get_len (
+				GO_DATA_VECTOR (series->base.values[2].data));
+			if (n > tmp)
+				n = tmp;
+		}
 		if (GOG_IS_BUBBLE_PLOT (model)) {
 			double zmin;
 			go_data_vector_get_minmax (GO_DATA_VECTOR (series->base.values[2].data), &zmin, &zmax);
@@ -696,23 +967,110 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 					gog_axis_map_to_view (y_map, y_vals[i]):
 					go_nan;
 			}
-			if (GOG_XY_PLOT (view->model)->use_splines) {
-				ArtBpath *path;
-				path = go_line_build_bpath (x_splines, y_splines, n);
-				gog_renderer_draw_bezier_path (view->renderer, path, &view->residual);
-				art_free (path);
-			} else {
+			switch (style->interpolation.type) {
+			case GO_LINE_INTERPOLATION_LINEAR: {
 				ArtVpath *path;
 				path = go_line_build_vpath (x_splines, y_splines, n);
-				gog_renderer_draw_path (view->renderer, path, &view->residual);
+				gog_renderer_draw_path (view->renderer, path);
 				art_free (path);
+				break;
+			}
+			case GO_LINE_INTERPOLATION_SPLINE: {
+				ArtBpath *path;
+				path = go_line_build_bpath (x_splines, y_splines, n);
+				gog_renderer_draw_bezier_path (view->renderer, path);
+				art_free (path);
+				break;
+			}
+			case GO_LINE_INTERPOLATION_STEP_START:
+			case GO_LINE_INTERPOLATION_STEP_END:
+			case GO_LINE_INTERPOLATION_STEP_CENTER_X:
+			case GO_LINE_INTERPOLATION_STEP_CENTER_Y: {
+				unsigned i, j;
+				ArtVpath *path;
+				gboolean b = FALSE;
+				i = 0;
+				while ((!go_finite (x_splines[i]) || !go_finite (y_splines[i]))  && n < i)
+					i++;
+				if (n == i)
+					break;
+				path = art_new (ArtVpath,
+					((style->interpolation.type <= GO_LINE_INTERPOLATION_STEP_END)?
+					2 * n + 1: 3 * n + 1));
+				path[0].code = ART_MOVETO;
+				path[0].x = x_splines[i];
+				path[0].y = y_splines[i++];
+				for (j = 1; i < n; i++) {
+					while ((!go_finite (x_splines[i]) ||
+							!go_finite (y_splines[i]))  && i < n) {
+						i++;
+						b = TRUE;
+					}
+					if (n == i)
+						break;
+					if (b) {
+						path[j].code = ART_MOVETO;
+						path[j].x = x_splines[i];
+						path[j++].y = y_splines[i];
+						b = FALSE;
+					} else switch (style->interpolation.type) {
+					case GO_LINE_INTERPOLATION_STEP_START:
+						path[j].code = ART_LINETO;
+						path[j].x = x_splines[i];
+						path[j++].y = y_splines[i-1];
+						path[j].code = ART_LINETO;
+						path[j].x = x_splines[i];
+						path[j++].y = y_splines[i];
+						break;
+					case GO_LINE_INTERPOLATION_STEP_END:
+						path[j].code = ART_LINETO;
+						path[j].x = x_splines[i-1];
+						path[j++].y = y_splines[i];
+						path[j].code = ART_LINETO;
+						path[j].x = x_splines[i];
+						path[j++].y = y_splines[i];
+						break;
+					case GO_LINE_INTERPOLATION_STEP_CENTER_X:
+						path[j].code = ART_LINETO;
+						path[j].x = path[j+1].x = (x_splines[i-1] + x_splines[i]) / 2.;
+						path[j++].y = y_splines[i-1];
+						path[j].code = ART_LINETO;
+						path[j++].y = y_splines[i];
+						path[j].code = ART_LINETO;
+						path[j].x = x_splines[i];
+						path[j++].y = y_splines[i];
+						break;
+					case GO_LINE_INTERPOLATION_STEP_CENTER_Y:
+						path[j].code = ART_LINETO;
+						path[j].x = x_splines[i-1];
+						path[j++].y = (y_splines[i-1] + y_splines[i]) / 2.;
+						path[j].code = ART_LINETO;
+						path[j].y = path[j-1].y;
+						path[j++].x = x_splines[i];
+						path[j].code = ART_LINETO;
+						path[j].x = x_splines[i];
+						path[j++].y = y_splines[i];
+						break;
+					default:
+						g_assert_not_reached ();
+						break;
+					}
+				}
+				path[j].code = ART_END;
+				gog_renderer_draw_sharp_path (view->renderer, path);
+				art_free (path);
+				break;
+			}
+			default:
+				g_assert_not_reached ();
+				break;
 			}
 			g_free (x_splines);
 			g_free (y_splines);
 		}
 
 		if (show_marks && !GOG_IS_BUBBLE_PLOT (model))
-			markers[j] = g_new (MarkerData, n);
+			markers[j] = g_new0 (MarkerData, n);
 
 		margin = gog_renderer_line_size (view->renderer, 1);
 		x_margin_min = view->allocation.x - margin;
@@ -726,13 +1084,15 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 			y = *y_vals++;
 			if (isnan (y) || isnan (x))
 				continue;
+			if (is_map && isnan (z = *z_vals++))
+				continue;
 			/* We are checking with go_finite here because isinf
 			   if not available everywhere.  Note, that NANs
 			   have been ruled out.  */
 			if (!gog_axis_map_finite (y_map, y))
-				y = 0; /* excel is just sooooo consistent */
+				y = (series->invalid_as_zero)? 0: go_nan; /* excel is just sooooo consistent */
 			if (!gog_axis_map_finite (x_map, x))
-				x = i;
+				x =  (series->invalid_as_zero)? 0: go_nan;
 			x_canvas = gog_axis_map_to_view (x_map, x);
 			y_canvas = gog_axis_map_to_view (y_map, y);
 			if (GOG_IS_BUBBLE_PLOT(model)) {
@@ -778,6 +1138,10 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 			    y_margin_min <= y_canvas && y_canvas <= y_margin_max) {
 				markers[j][k].x = x_canvas;
 				markers[j][k].y = y_canvas;
+				if (is_map)
+					markers[j][k].color = (gog_axis_map_finite (z_map, z))?
+							get_map_color (gog_axis_map_to_view (z_map, z)):
+							0;
 				k++;
 			}
 		}
@@ -793,7 +1157,7 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 			g_object_unref (neg_style);
 	}
 
-	gog_renderer_clip_pop (view->renderer);
+	gog_renderer_pop_clip (view->renderer);
 
 	if (!GOG_IS_BUBBLE_PLOT (model))
 		for (j = 0, ptr = model->base.series ; ptr != NULL ; ptr = ptr->next, j++) {
@@ -801,31 +1165,32 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 					series = ptr->data;
 					style = GOG_STYLED_OBJECT (series)->style;
 					gog_renderer_push_style (view->renderer, style);
-					for (k = 0; k < num_markers[j]; k++)
+					for (k = 0; k < num_markers[j]; k++) {
+						if (is_map) {
+							go_marker_set_outline_color
+								(style->marker.mark,markers[j][k].color);
+							go_marker_set_fill_color
+								(style->marker.mark,markers[j][k].color);
+							gog_renderer_push_style (view->renderer, style);
+						}
 						gog_renderer_draw_marker (view->renderer, 
 									  markers[j][k].x,
 									  markers[j][k].y);
+						if (is_map)
+							gog_renderer_pop_style (view->renderer);
+					}
 					gog_renderer_pop_style (view->renderer);
 					g_free (markers[j]);
 				}
 			}
 
 	/* Now render children, may be should come before markers? */
-	gog_renderer_clip_push (view->renderer, &view->residual);
 	for (ptr = view->children ; ptr != NULL ; ptr = ptr->next)
 		gog_view_render	(ptr->data, bbox);
-	gog_renderer_clip_pop (view->renderer);
 
-	gog_axis_map_free (x_map);
-	gog_axis_map_free (y_map);
-}
-
-static gboolean
-gog_xy_view_info_at_point (GogView *view, double x, double y,
-			   GogObject const *cur_selection,
-			   GogObject **obj, char **name)
-{
-	return FALSE;
+	gog_chart_map_free (chart_map);
+	if (is_map)
+		gog_axis_map_free (z_map);
 }
 
 static GogViewClass *xy_view_parent_klass;
@@ -845,7 +1210,6 @@ gog_xy_view_class_init (GogViewClass *view_klass)
 	xy_view_parent_klass = (GogViewClass*) g_type_class_peek_parent (view_klass);
 	view_klass->render	  = gog_xy_view_render;
 	view_klass->size_allocate = gog_xy_view_size_allocate;
-	view_klass->info_at_point = gog_xy_view_info_at_point;
 	view_klass->clip	  = FALSE;
 }
 
@@ -885,12 +1249,61 @@ gog_xy_series_view_class_init (GogXYSeriesViewClass *gview_klass)
 	GogViewClass *view_klass = GOG_VIEW_CLASS (gview_klass);
 	view_klass->render = gog_xy_series_view_render;
 	view_klass->size_allocate = gog_xy_series_view_size_allocate;
+	view_klass->build_toolkit = NULL;
 }
 
 GSF_DYNAMIC_CLASS (GogXYSeriesView, gog_xy_series_view,
 	gog_xy_series_view_class_init, NULL,
 	GOG_VIEW_TYPE)
 
+/*****************************************************************************/
+
+static gboolean
+horiz_drop_lines_can_add (GogObject const *parent)
+{
+	GogXYSeries *series = GOG_XY_SERIES (parent);
+	return (series->hdroplines == NULL);
+}
+
+static void
+horiz_drop_lines_post_add (GogObject *parent, GogObject *child)
+{
+	GogXYSeries *series = GOG_XY_SERIES (parent);
+	series->hdroplines = child;
+	gog_object_request_update (child);
+}
+
+static void
+horiz_drop_lines_pre_remove (GogObject *parent, GogObject *child)
+{
+	GogXYSeries *series = GOG_XY_SERIES (parent);
+	series->hdroplines = NULL;
+}
+
+/*****************************************************************************/
+
+static gboolean
+vert_drop_lines_can_add (GogObject const *parent)
+{
+	GogXYSeries *series = GOG_XY_SERIES (parent);
+	return (series->vdroplines == NULL);
+}
+
+static void
+vert_drop_lines_post_add (GogObject *parent, GogObject *child)
+{
+	GogXYSeries *series = GOG_XY_SERIES (parent);
+	series->vdroplines = child;
+	gog_object_request_update (child);
+}
+
+static void
+vert_drop_lines_pre_remove (GogObject *parent, GogObject *child)
+{
+	GogXYSeries *series = GOG_XY_SERIES (parent);
+	series->vdroplines = NULL;
+}
+
 /****************************************************************************/
 
 typedef GogSeriesClass GogXYSeriesClass;
@@ -898,7 +1311,8 @@ typedef GogSeriesClass GogXYSeriesClass;
 enum {
 	SERIES_PROP_0,
 	SERIES_PROP_XERRORS,
-	SERIES_PROP_YERRORS
+	SERIES_PROP_YERRORS,
+	SERIES_PROP_INVALID_AS_ZERO
 };
 
 static GogStyledObjectClass *series_parent_klass;
@@ -917,7 +1331,7 @@ gog_xy_series_update (GogObject *obj)
 		y_len = go_data_vector_get_len (
 			GO_DATA_VECTOR (series->base.values[1].data));
 	}
-	if (GOG_IS_BUBBLE_PLOT (series->base.plot)) {
+	if (GOG_IS_BUBBLE_PLOT (series->base.plot) || GOG_IS_XY_COLOR_PLOT (series->base.plot)) {
 		double *z_vals = NULL;
 		int z_len = 0;
 		if (series->base.values[2].data != NULL) {
@@ -937,7 +1351,8 @@ gog_xy_series_update (GogObject *obj)
 
 	/* update children */
 	for (ptr = obj->children; ptr != NULL; ptr = ptr->next)
-		gog_object_request_update (GOG_OBJECT (ptr->data));
+		if (!IS_GOG_SERIES_LINES (ptr->data))
+			gog_object_request_update (GOG_OBJECT (ptr->data));
 
 	/* queue plot for redraw */
 	gog_object_request_update (GOG_OBJECT (series->base.plot));
@@ -955,7 +1370,8 @@ gog_xy_series_init (GObject *obj)
 
 	series->x_errors = series->y_errors = NULL;
 	(GOG_SERIES (series))->acceptable_children =
-				GOG_SERIES_ACCEPT_REGRESSION_CURVE;
+				GOG_SERIES_ACCEPT_TREND_LINE;
+	series->hdroplines = series->vdroplines = NULL;
 }
 
 static void
@@ -980,22 +1396,34 @@ static void
 gog_xy_series_init_style (GogStyledObject *gso, GogStyle *style)
 {
 	GogSeries *series = GOG_SERIES (gso);
-	GogXYPlot const *plot;
 
 	series_parent_klass->init_style (gso, style);
 	if (series->plot == NULL ||
 	    GOG_IS_BUBBLE_PLOT (series->plot))
 		return;
 
-	plot = GOG_XY_PLOT (series->plot);
+	if (GOG_IS_XY_PLOT (series->plot)) {
+		GogXYPlot const *plot = GOG_XY_PLOT (series->plot);
 
-	if (!plot->default_style_has_markers &&
-	    style->marker.auto_shape) 
-		go_marker_set_shape (style->marker.mark, GO_MARKER_NONE);
+		if (!plot->default_style_has_markers &&
+			style->marker.auto_shape) 
+			go_marker_set_shape (style->marker.mark, GO_MARKER_NONE);
 
-	if (!plot->default_style_has_lines &&
-	    style->line.auto_dash)
-		style->line.dash_type = GO_LINE_NONE;
+		if (!plot->default_style_has_lines &&
+			style->line.auto_dash)
+			style->line.dash_type = GO_LINE_NONE;
+
+		if (style->interpolation.auto_type)
+			style->interpolation.type = plot->interpolation;
+	} else {
+		GogXYColorPlot const *plot = GOG_XY_COLOR_PLOT (series->plot);
+		if (!plot->default_style_has_lines &&
+			style->line.auto_dash)
+			style->line.dash_type = GO_LINE_NONE;
+
+		if (style->interpolation.auto_type)
+			style->interpolation.type = plot->interpolation;
+	}
 }
 
 static void
@@ -1042,6 +1470,12 @@ gog_xy_series_set_property (GObject *obj, guint param_id,
 			g_object_unref (series->y_errors);
 		series->y_errors = bar;
 		break;
+	case SERIES_PROP_INVALID_AS_ZERO:
+		series->invalid_as_zero = g_value_get_boolean (value);
+		gog_object_request_update (GOG_OBJECT (series));
+		break;
+	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
+		 break;
 	}
 }
 
@@ -1058,28 +1492,79 @@ gog_xy_series_get_property (GObject *obj, guint param_id,
 	case SERIES_PROP_YERRORS :
 		g_value_set_object (value, series->y_errors);
 		break;
+	case SERIES_PROP_INVALID_AS_ZERO:
+		g_value_set_boolean (value, series->invalid_as_zero);
+		break;
+	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
+		 break;
 	}
 }
 
+#ifdef GOFFICE_WITH_GTK
+#include <goffice/gtk/goffice-gtk.h>
+#include <gtk/gtktogglebutton.h>
+static void
+invalid_toggled_cb (GtkToggleButton *btn, GObject *obj)
+{
+	g_object_set (obj, "invalid-as-zero", gtk_toggle_button_get_active (btn), NULL);
+}
+
 static void 
 gog_xy_series_populate_editor (GogObject *obj,
 			       GogEditor *editor,
-				GogDataAllocator *dalloc,
-				GOCmdContext *cc)
+			       GogDataAllocator *dalloc,
+			       GOCmdContext *cc)
 {
-	GtkWidget *error_page;
+	GtkWidget *w;
+	char const *dir = go_plugin_get_dir_name (
+		go_plugins_get_plugin_by_id ("GOffice_plot_xy"));
+	char *path = g_build_filename (dir, "gog-xy-series-prefs.glade", NULL);
+	GladeXML *gui = go_libglade_new (path, "gog-xy-series-prefs", GETTEXT_PACKAGE, cc);
 
+	g_free (path);
 	(GOG_OBJECT_CLASS(series_parent_klass)->populate_editor) (obj, editor, dalloc, cc);
-	
-	error_page = gog_error_bar_prefs (GOG_SERIES (obj), "x-errors", TRUE, dalloc, cc);
-	gog_editor_add_page (editor, error_page, _("X error bars"));
-	error_page = gog_error_bar_prefs (GOG_SERIES (obj), "y-errors", FALSE, dalloc, cc);
-	gog_editor_add_page (editor, error_page, _("Y error bars"));
+
+	if (gui != NULL) {
+		w = glade_xml_get_widget (gui, "invalid-as-zero");
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+				(GOG_XY_SERIES (obj))->invalid_as_zero);
+		g_signal_connect (G_OBJECT (w),
+			"toggled",
+			G_CALLBACK (invalid_toggled_cb), obj);
+		w = glade_xml_get_widget (gui, "gog-xy-series-prefs");
+		g_object_set_data_full (G_OBJECT (w),
+			"state", gui, (GDestroyNotify)g_object_unref);
+		gog_editor_add_page (editor, w, _("Details"));
+	}
+
+	w = gog_error_bar_prefs (GOG_SERIES (obj), "x-errors", TRUE, dalloc, cc);
+	gog_editor_add_page (editor, w, _("X error bars"));
+	w = gog_error_bar_prefs (GOG_SERIES (obj), "y-errors", FALSE, dalloc, cc);
+	gog_editor_add_page (editor, w, _("Y error bars"));
 }
+#endif
 
 static void
 gog_xy_series_class_init (GogStyledObjectClass *gso_klass)
 {
+	static GogObjectRole const roles[] = {
+		{ N_("Horizontal drop lines"), "GogSeriesLines", 2,
+			GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
+			horiz_drop_lines_can_add,
+			NULL,
+			NULL,
+			horiz_drop_lines_post_add,
+			horiz_drop_lines_pre_remove,
+			NULL },
+		{ N_("Vertical drop lines"), "GogSeriesLines",	3,
+			GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
+			vert_drop_lines_can_add,
+			NULL,
+			NULL,
+			vert_drop_lines_post_add,
+			vert_drop_lines_pre_remove,
+			NULL },
+	};
 	GogObjectClass *gog_klass = (GogObjectClass *)gso_klass;
 	GObjectClass *gobject_klass = (GObjectClass *) gso_klass;
 
@@ -1092,9 +1577,13 @@ gog_xy_series_class_init (GogStyledObjectClass *gso_klass)
 	gobject_klass->set_property = gog_xy_series_set_property;
 	gobject_klass->get_property = gog_xy_series_get_property;
 	gog_klass->update		= gog_xy_series_update;
+#ifdef GOFFICE_WITH_GTK
 	gog_klass->populate_editor	= gog_xy_series_populate_editor;
+#endif
 	gso_klass->init_style		= gog_xy_series_init_style;
 
+	gog_object_register_roles (gog_klass, roles, G_N_ELEMENTS (roles));
+
 	g_object_class_install_property (gobject_klass, SERIES_PROP_XERRORS,
 		g_param_spec_object ("x-errors", "x-errors",
 			"GogErrorBar *",
@@ -1103,6 +1592,10 @@ gog_xy_series_class_init (GogStyledObjectClass *gso_klass)
 		g_param_spec_object ("y-errors", "y-errors",
 			"GogErrorBar *",
 			GOG_ERROR_BAR_TYPE, G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
+	g_object_class_install_property (gobject_klass, SERIES_PROP_INVALID_AS_ZERO,
+		g_param_spec_boolean ("invalid-as-zero", _("invalid-as-zero"),
+			_("Replace invalid values by 0 when drawing markers or bubbles"),
+			FALSE, G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
 }
 
 GSF_DYNAMIC_CLASS (GogXYSeries, gog_xy_series,
@@ -1116,6 +1609,7 @@ go_plugin_init (GOPlugin *plugin, GOCmdContext *cc)
 	gog_2d_plot_register_type (module);
 	gog_xy_plot_register_type (module);
 	gog_bubble_plot_register_type (module);
+	gog_xy_color_plot_register_type (module);
 	gog_xy_view_register_type (module);
 	gog_xy_series_view_register_type (module);
 	gog_xy_series_register_type (module);
